<?php

if (!defined('ABSPATH')) exit;

class sap_admin {
    public $page_ids = array();
    public $thumb_size = 64;

    function __construct() {
        add_action('wp_ajax_sap_tracks_log', array(&$this, 'ajax_tracks_log'));

        add_filter('post_row_actions', array(&$this, 'row_actions'), 10, 2);
        add_action('admin_init', array(&$this, 'save_settings'));
        add_action('admin_menu', array(&$this, 'admin_meta'));
        add_action('admin_menu', array(&$this, 'admin_menu'));
        add_filter('manage_sapplaylist_posts_columns', array(&$this, 'add_columns'));
        add_action('manage_sapplaylist_posts_custom_column', array(&$this, 'columns_data'), 10, 2);
        add_filter('set-screen-option', array(&$this, 'screen_options_grid_rows_save'), 10, 3);

        add_filter('plugin_row_meta', array(&$this, 'plugin_row_meta'), 10, 2);
        add_filter('plugin_action_links_smart-audio-playlists/smart-audio-playlists.php', array(&$this, 'plugin_action_links'));

        add_action('admin_enqueue_scripts', array(&$this, 'admin_enqueue_scripts'));

        add_action('save_post_sapplaylist', array(&$this, 'save_playlist'), 10, 3);

        add_action('admin_notices', array(&$this, 'info_admin_notice'));
    }

    public function ajax_tracks_log() {
        $id = isset($_GET['id']) ? intval($_GET['id']) : 0;
        $playlist = isset($_GET['playlist']) ? intval($_GET['playlist']) : 0;

        if ($id == 0 || $playlist == 0) {
            echo '<h1>'.__("Error", "smart-audio-playlists").'</h1>';
            echo __("Invalid Playlist Requested.", "smart-audio-playlists");
        } else {
            require_once(SAP_PATH.'core/grids/tracks.php');

            echo '<h1>'.get_post($playlist)->post_title.'</h1>';

            $_grid = new sap_tracks_log($id);
            $_grid->playlist = $id;
            $_grid->prepare_items();
            $_grid->display();
        }

        die('');
    }

    public function row_actions($actions, $post) {
        if ($post->post_type == 'sapplaylist') {
            $actions['saplog'] = sprintf('<a href="edit.php?post_type=sapplaylist&page=smart-audio-playlists-log&filter-list=%s">%s</a>', $post->ID, __("Playtime Log", "smart-audio-playlists"));
        }

        return $actions;
    }

    public function screen_options_grid_rows_save($status, $option, $value) {
        if ($option == 'sap_rows_per_page_log') {
            return $value;
        }

        return $status;
    }

    public function screen_options_grid_rows() {
        $args = array(
            'label' => __("Rows", "smart-audio-playlists"),
            'default' => 25,
            'option' => 'sap_rows_per_page_log'
        );

        add_screen_option('per_page', $args);
    }

    public function info_admin_notice() {
        global $pagenow;

        if (!current_theme_supports('post-thumbnails', 'sapplaylist')) {
            if ($pagenow == 'post-new.php' || $pagenow == 'edit.php' || $pagenow == 'post.php') {
                global $post;

                $post_type = isset($_GET['post_type']) && $_GET['post_type'] != '' ? $_GET['post_type'] : $post->post_type;

                if ($post_type == 'sapplaylist') {
                    echo '<div class="updated error"><p>';
                    _e("Current theme doesn't support Post Thumbnails feature. For this plugin to work properly, this needs to be enable.", "smart-audio-playlists");
                    echo '<br/>';
                    _e("You can force post thumbnails feature registration", "smart-audio-playlists");
                    echo ': <a href="edit.php?post_type=sapplaylist&page=smart-audio-playlists-advanced#sap___force_post_thumbnails">'.__("Enable Post Thumbnails", "smart-audio-playlists").'</a>';
                    echo '</p></div>';
                }
            }
        }

        if (smart_sap_core()->install_or_update()) {
            if ($pagenow == 'post-new.php' || $pagenow == 'edit.php' || $pagenow == 'post.php') {
                global $post;

                $post_type = isset($_GET['post_type']) && $_GET['post_type'] != '' ? $_GET['post_type'] : $post->post_type;

                if ($post_type == 'sapplaylist') {
                    echo '<div class="updated"><p>';
                    if (smart_sap_core()->get('show_install_screen')) {
                        _e("Plugin is now installed. You should review the installation process and check if everything is OK.", "smart-audio-playlists");
                    } else {
                        _e("Plugin is now updated. You should review the update process and check if everything is OK.", "smart-audio-playlists");
                    }

                    echo '<br/>';
                    _e("Click here to", "smart-audio-playlists");
                    echo ': <a href="edit.php?post_type=sapplaylist&page=smart-audio-playlists-settings">';

                    if (smart_sap_core()->get('show_install_screen')) {
                        _e("Review Installation Process", "smart-audio-playlists");
                    } else {
                        _e("Review Update Process", "smart-audio-playlists");
                    }

                    echo '</a>';
                    echo '</p></div>';
                }
            }
        }
    }

    public function add_columns($columns) {
        $new_columns = array();

        $i = 0;
        foreach ($columns as $key => $value) {
            if ($i == 1) {
                $new_columns['sap_column_id'] = 'ID';
            }

            if ($key == 'title') {
                $new_columns['sap_column_cover'] = __("Cover", "smart-audio-playlists");
            }

            if ($key == 'date') {
                $new_columns['sap_column_tracks'] = '<span class="dashicons dashicons-media-audio" title="'.__("Tracks", "smart-audio-playlists").'"></span>';
            }

            if ($key == 'date') {
                $new_columns['sap_column_time'] = '<span class="dashicons dashicons-clock" title="'.__("Time", "smart-audio-playlists").'"></span>';
            }

            $new_columns[$key] = $value;
            $i++;
        }

        return $new_columns;
    }

    public function columns_data($column, $id) {
        switch ($column) {
            case 'sap_column_id':
                echo $id;
                break;
            case 'sap_column_time':
                echo sap_playlist($id)->running_time();
                break;
            case 'sap_column_tracks':
                echo count(sap_playlist($id)->tracks);
                break;
            case 'sap_column_cover':
                $thumbnail = get_the_post_thumbnail($id, array($this->thumb_size, $this->thumb_size));

                if ($thumbnail == '') {
                    _e("None", "smart-audio-playlists");
                } else {
                    echo $thumbnail;
                }
                break;
        }
    }

    public function plugin_row_meta($links, $plugin_file) {
        if ($plugin_file == 'smart-audio-playlists/smart-audio-playlists.php') {
            $links[] = 'SMART Plugins: <a href="http://www.smartplugins.info/" target="_blank">Website</a>';
            $links[] = '<a href="http://codecanyon.net/user/GDragoN/portfolio?ref=GDragoN" target="_blank">On CodeCanyon</a>';
        }

        return $links;
    }

    public function plugin_action_links($links) {
        $links[] = '<a href="edit.php?post_type=sapplaylist">'.__("Playlists", "smart-audio-playlists").'</a>';
        $links[] = '<a href="edit.php?post_type=sapplaylist&page=smart-audio-playlists-settings">'.__("Settings", "smart-audio-playlists").'</a>';

	return $links;
    }

    public function admin_enqueue_scripts($hook) {
        if (substr($hook, 0, 39) == 'sapplaylist_page_smart-audio-playlists-') {
            wp_enqueue_script('jquery');

            $depend = array('jquery', 'sap-jqueryui');

            wp_enqueue_script('sap-jqueryui', SAP_URL.'js/jquery-ui.js', array('jquery'), smart_sap_core()->get('__version__'), true);
            wp_enqueue_style('sap-jqueryui', SAP_URL.'css/smoothness/jquery-ui.css', array(), smart_sap_core()->get('__version__'));

            wp_enqueue_script('sap-admin', (defined('SCRIPT_DEBUG') && SCRIPT_DEBUG ? SAP_URL.'js/admin.js' : SAP_URL.'js/admin.min.js'), $depend, smart_sap_core()->get('__version__'), true);
            wp_enqueue_style('sap-admin', SAP_URL.'css/admin.css', array(), smart_sap_core()->get('__version__'));

            wp_localize_script('sap-admin', 'sap_admin_data', array(
                'string_log_loading' => __("Loading, please wait...", "smart-audio-playlists"),
                'string_are_you_sure' => __("Are you sure you want to do this?", "smart-audio-playlists")
            ));

            do_action('sap_admin_enqueue_scripts');
        }

        if ($hook == 'post-new.php' || $hook == 'edit.php' || $hook == 'post.php') {
            global $post;

            $post_type = isset($_GET['post_type']) && $_GET['post_type'] != '' ? $_GET['post_type'] : $post->post_type;

            if ($post_type == 'sapplaylist') {
                wp_enqueue_style('sap-meta', SAP_URL.'css/meta.css', array(), smart_sap_core()->get('__version__'));
                wp_enqueue_script('sap-meta', (defined('SCRIPT_DEBUG') && SCRIPT_DEBUG ? SAP_URL.'js/meta.js' : SAP_URL.'js/meta.min.js'), array(), smart_sap_core()->get('__version__'));

                wp_localize_script('sap-meta', 'sap_meta_data', array(
                    'current_panel' => $hook == 'edit.php' ? 'playlist' : 'editor',
                    'string_various_artists' => __("Various Artists", "smart-audio-playlists"),
                    'string_compilation_playlist' => __("Compilation Playlist", "smart-audio-playlists"),
                    'string_dialog_confirm_remove' => __("Are you sure you want to remove this track?", "smart-audio-playlists"),
                    'string_dialog_confirm_clear' => __("Are you sure you want to remove all tracks?", "smart-audio-playlists"),
                    'string_media_thumb_title' => __("Assign Image to Track", "smart-audio-playlists"),
                    'string_media_thumb_button' => __("Use selected Image", "smart-audio-playlists"),
                    'string_media_tracks_title' => __("Add tracks to Playlist", "smart-audio-playlists"),
                    'string_media_tracks_button' => __("Use in Playlist", "smart-audio-playlists")
                ));
            }
        }
    }

    public function admin_meta() {
        add_meta_box('sap-playlist-information', __("Instructions to create Playlist", "smart-audio-playlists"), array(&$this, 'meta_playlist_info'), 'sapplaylist', 'advanced', 'high');
        add_meta_box('sap-playlist-tracks', __("Playlist Tracks", "smart-audio-playlists"), array(&$this, 'meta_playlist_tracks'), 'sapplaylist', 'advanced', 'high');
        add_meta_box('sap-playlist-about', __("Playlist Description", "smart-audio-playlists"), array(&$this, 'meta_playlist_about'), 'sapplaylist', 'advanced', 'low');
        add_meta_box('sap-playlist-heading', __("Playlist Heading", "smart-audio-playlists"), array(&$this, 'meta_playlist_heading'), 'sapplaylist', 'advanced');
        add_meta_box('sap-playlist-meta', __("Playlist Meta Data", "smart-audio-playlists"), array(&$this, 'meta_playlist_meta'), 'sapplaylist', 'side', 'high');
        add_meta_box('sap-playlist-details', __("Playlist Information", "smart-audio-playlists"), array(&$this, 'meta_playlist_details'), 'sapplaylist', 'side', 'high');
        add_meta_box('sap-playlist-settings', __("Playlist Settings", "smart-audio-playlists"), array(&$this, 'meta_playlist_settings'), 'sapplaylist', 'side', 'high');
        add_meta_box('sap-playlist-stats', __("Playlist Statistics", "smart-audio-playlists"), array(&$this, 'meta_playlist_stats'), 'sapplaylist', 'side', 'high');
    }

    public function admin_menu() {
        $this->page_ids[] = add_submenu_page('edit.php?post_type=sapplaylist', __("Smart Audio Playlists: Settings", "smart-audio-playlists"), __("Settings", "smart-audio-playlists"), 'activate_plugins', 'smart-audio-playlists-settings', array(&$this, 'menu_settings'));
        $this->page_ids[] = add_submenu_page('edit.php?post_type=sapplaylist', __("Smart Audio Playlists: Theme", "smart-audio-playlists"), __("Theme", "smart-audio-playlists"), 'activate_plugins', 'smart-audio-playlists-theme', array(&$this, 'menu_settings'));
        $this->page_ids[] = add_submenu_page('edit.php?post_type=sapplaylist', __("Smart Audio Playlists: Advanced", "smart-audio-playlists"), __("Advanced", "smart-audio-playlists"), 'activate_plugins', 'smart-audio-playlists-advanced', array(&$this, 'menu_settings'));
        $this->page_ids[] = add_submenu_page('edit.php?post_type=sapplaylist', __("Smart Audio Playlists: Log", "smart-audio-playlists"), __("Log", "smart-audio-playlists"), 'activate_plugins', 'smart-audio-playlists-log', array(&$this, 'menu_settings'));
        $this->page_ids[] = add_submenu_page('edit.php?post_type=sapplaylist', __("Smart Audio Playlists: About", "smart-audio-playlists"), __("About", "smart-audio-playlists"), 'activate_plugins', 'smart-audio-playlists-about', array(&$this, 'menu_about'));

        add_action('load-sapplaylist_page_smart-audio-playlists-log', array(&$this, 'screen_options_grid_rows'));

        foreach ($this->page_ids as $id) {
            add_action('load-'.$id, array(&$this, 'load_admin_page_shared'));
        }
    }

    public function load_admin_page_shared() {
        $screen = get_current_screen();

        $screen->set_help_sidebar('
            <p><strong>SMART Plugins:</strong></p>
            <p><a target="_blank" href="http://www.smartplugins.info/">'.__("Website", "smart-audio-playlists").'</a></p>
            <p><a target="_blank" href="http://codecanyon.net/user/GDragoN/portfolio?ref=GDragoN">'.__("On CodeCanyon", "smart-audio-playlists").'</a></p>
            <p><a target="_blank" href="http://twitter.com/millanrs">'.__("On Twitter", "smart-audio-playlists").'</a></p>
            <p><a target="_blank" href="http://facebook.com/smartplugins">'.__("On Facebook", "smart-audio-playlists").'</a></p>');

        $screen->add_help_tab(array(
            'id' => 'scs-screenhelp-info',
            'title' => __("Information", "smart-audio-playlists"),
            'content' => '<p>'.__("Add newsletter subscription forms to the website using widgets, shortcodes or functions with support for many subscription services.", "smart-audio-playlists").'</p>
                <h5>'.__("Useful Links", "smart-audio-playlists").'</h5>
                <p><a target="_blank" href="http://www.smartplugins.info/plugin/wordpress/smart-audio-playlists/">'.__("Plugin Homepage", "smart-audio-playlists").'</a></p>
                <p><a target="_blank" href="http://d4p.me/ccsap">'.__("Plugin On CodeCanyon", "smart-audio-playlists").'</a></p>'
        ));

        $screen->add_help_tab(array(
            'id' => 'scs-screenhelp-support',
            'title' => __("Support", "smart-audio-playlists"),
            'content' => '<h5>'.__("Support Reources", "smart-audio-playlists").'</h5>
                <p><a target="_blank" href="http://forum.smartplugins.info/forums/forum/smart/smart-ajax-subscribe/">'.__("Official Support Forum", "smart-audio-playlists").'</a></p>'
        ));
    }

    public function action_delete_playlist_log($ids) {
        global $wpdb;

        $ids = (array)$ids;

        foreach ($ids as $id) {
            $id = intval($id);

            if ($id > 0) {
                $wpdb->delete($wpdb->prefix.'sap_play_log', array('id' => $id));
                $wpdb->delete($wpdb->prefix.'sap_play_log', array('parent_id' => $id));
            }
        }
    }

    public function save_settings() {
        do_action('sap_admin_save_settings');

        if (isset($_GET['page']) && $_GET['page'] == 'smart-audio-playlists-log') {
            if (isset($_GET['single-action'])) {
                $nonce = isset($_GET['_wpnonce']) ? $_GET['_wpnonce'] : '';

                if (wp_verify_nonce($nonce, 'smart-audio-playlists-log') !== false) {
                    $action = $_GET['single-action'];
                    $id = isset($_GET['log']) ? $_GET['log'] : 0;

                    if ($action == 'delete') {
                        $this->action_delete_playlist_log($id);
                    }

                    wp_redirect('edit.php?post_type=sapplaylist&page=smart-audio-playlists-log&delete-done=true');
                    exit;
                }
            }

            if (isset($_GET['action']) || isset($_GET['action2'])) {
                check_admin_referer('bulk-playlists');

                $action = isset($_GET['action']) && $_GET['action'] != '' && $_GET['action'] != '-1' ? $_GET['action'] : '';

                if ($action == '') {
                    $action = isset($_GET['action2']) && $_GET['action2'] != '' && $_GET['action2'] != '-1' ? $_GET['action2'] : '';
                }

                if ($action != '') {
                    $ids = isset($_GET['playlist']) ? (array)$_GET['playlist'] : array();

                    if (!empty($ids) && $action == 'delete') {
                        $this->action_delete_playlist_log($ids);
                    }

                    wp_redirect('edit.php?post_type=sapplaylist&page=smart-audio-playlists-log&delete-done=true');
                    exit;
                }
            }
        }

        if (isset($_POST['option_page']) && $_POST['option_page'] == 'smart-audio-playlists-settings') {
            check_admin_referer('smart-audio-playlists-settings-options');

            $sap = $_POST['sap'];

            smart_sap_core()->set('player_style', strip_tags(stripslashes($sap['player_style'])));
            smart_sap_core()->set('player_skin', strip_tags(stripslashes($sap['player_skin'])));

            smart_sap_core()->set('track_use_playlist_image', isset($sap['track_use_playlist_image']));
            smart_sap_core()->set('track_use_default_image', isset($sap['track_use_default_image']));

            smart_sap_core()->set('hide_playlist_statistics', isset($sap['hide_playlist_statistics']));
            smart_sap_core()->set('hide_tracks_statistics', isset($sap['hide_tracks_statistics']));

            smart_sap_core()->set('player_prevtrack', isset($sap['player_prevtrack']));
            smart_sap_core()->set('player_nexttrack', isset($sap['player_nexttrack']));
            smart_sap_core()->set('player_current', isset($sap['player_current']));
            smart_sap_core()->set('player_progress', isset($sap['player_progress']));
            smart_sap_core()->set('player_duration', isset($sap['player_duration']));
            smart_sap_core()->set('player_volume', isset($sap['player_volume']));

            smart_sap_core()->set('player_native_ipad', isset($sap['player_native_ipad']));
            smart_sap_core()->set('player_native_iphone', isset($sap['player_native_iphone']));
            smart_sap_core()->set('player_native_android', isset($sap['player_native_android']));

            smart_sap_core()->set('player_visible_track_download', isset($sap['player_visible_track_download']));
            smart_sap_core()->set('player_visible_track_images', isset($sap['player_visible_track_images']));
            smart_sap_core()->set('player_visible_track_numbers', isset($sap['player_visible_track_numbers']));
            smart_sap_core()->set('player_visible_track_statistics', isset($sap['player_visible_track_statistics']));

            smart_sap_core()->set('player_visible_about', isset($sap['player_visible_about']));
            smart_sap_core()->set('player_visible_statistics', isset($sap['player_visible_statistics']));
            smart_sap_core()->set('player_visible_metadata', isset($sap['player_visible_metadata']));

            smart_sap_core()->set('player_toggle', isset($sap['player_toggle']));
            smart_sap_core()->set('player_toggle_hidden', isset($sap['player_toggle_hidden']));
            smart_sap_core()->set('player_toggle_nohide', isset($sap['player_toggle_nohide']));

            smart_sap_core()->set('player_tracklist_toggle_open', stripslashes($sap['player_tracklist_toggle_open']));
            smart_sap_core()->set('player_tracklist_toggle_close', stripslashes($sap['player_tracklist_toggle_close']));
            smart_sap_core()->set('player_link_playlist_text', stripslashes($sap['player_link_playlist_text']));

            $volume = intval($sap['player_initial_volume']);

            if ($volume > 100) {
                $volume = 100;
            } else if ($volume < 0) {
                $volume = 0;
            }

            smart_sap_core()->set('player_initial_volume', $volume);

            smart_sap_core()->save();

            flush_rewrite_rules();

            wp_redirect('edit.php?post_type=sapplaylist&page=smart-audio-playlists-settings&settings-updated=true');
            exit;
        }        

        if (isset($_POST['option_page']) && $_POST['option_page'] == 'smart-audio-playlists-theme') {
            check_admin_referer('smart-audio-playlists-theme-options');

            $sap = $_POST['sap'];

            smart_sap_core()->set('archive_playlists_count', intval($sap['archive_playlists_count']));

            smart_sap_core()->set('templates_override', isset($sap['templates_override']));
            smart_sap_core()->set('templates_playlist_single', strip_tags(stripslashes($sap['templates_playlist_single'])));
            smart_sap_core()->set('templates_single_title', isset($sap['templates_single_title']));
            smart_sap_core()->set('templates_playlist_archive', strip_tags(stripslashes($sap['templates_playlist_archive'])));
            smart_sap_core()->set('templates_archive_title', isset($sap['templates_archive_title']));
            smart_sap_core()->set('templates_playlist_index_post_tag', strip_tags(stripslashes($sap['templates_playlist_index_post_tag'])));
            smart_sap_core()->set('templates_playlist_index_sapgenre', strip_tags(stripslashes($sap['templates_playlist_index_sapgenre'])));
            smart_sap_core()->set('templates_playlist_index_sapartist', strip_tags(stripslashes($sap['templates_playlist_index_sapartist'])));
            smart_sap_core()->set('templates_index_title', isset($sap['templates_index_title']));

            smart_sap_core()->set('css_load_render', isset($sap['css_load_render']));
            smart_sap_core()->set('css_load_playlist', isset($sap['css_load_playlist']));
            smart_sap_core()->set('css_load_theme', isset($sap['css_load_theme']));

            smart_sap_core()->save();

            wp_redirect('edit.php?post_type=sapplaylist&page=smart-audio-playlists-theme&settings-updated=true');
            exit;
        }        

        if (isset($_POST['option_page']) && $_POST['option_page'] == 'smart-audio-playlists-advanced') {
            check_admin_referer('smart-audio-playlists-advanced-options');

            $sap = $_POST['sap'];

            smart_sap_core()->set('force_post_thumbnails', isset($sap['force_post_thumbnails']));

            smart_sap_core()->set('cpt_playlist_tax_tag', isset($sap['cpt_playlist_tax_tag']));
            smart_sap_core()->set('cpt_playlist_tax_genre', isset($sap['cpt_playlist_tax_genre']));
            smart_sap_core()->set('cpt_playlist_tax_artist', isset($sap['cpt_playlist_tax_artist']));

            smart_sap_core()->set('cpt_track_singular', strip_tags(stripslashes($sap['cpt_track_singular'])));
            smart_sap_core()->set('cpt_track_plural', strip_tags(stripslashes($sap['cpt_track_plural'])));
            smart_sap_core()->set('cpt_playlist_singular', strip_tags(stripslashes($sap['cpt_playlist_singular'])));
            smart_sap_core()->set('cpt_playlist_plural', strip_tags(stripslashes($sap['cpt_playlist_plural'])));
            smart_sap_core()->set('cpt_playlist_slug_single', strip_tags(stripslashes($sap['cpt_playlist_slug_single'])));
            smart_sap_core()->set('cpt_playlist_slug_archive', strip_tags(stripslashes($sap['cpt_playlist_slug_archive'])));

            smart_sap_core()->set('tax_genre_singular', strip_tags(stripslashes($sap['tax_genre_singular'])));
            smart_sap_core()->set('tax_genre_plural', strip_tags(stripslashes($sap['tax_genre_plural'])));
            smart_sap_core()->set('tax_genre_slug_single', strip_tags(stripslashes($sap['tax_genre_slug_single'])));
            smart_sap_core()->set('tax_genre_slug_archive', strip_tags(stripslashes($sap['tax_genre_slug_archive'])));
            smart_sap_core()->set('tax_artist_singular', strip_tags(stripslashes($sap['tax_artist_singular'])));
            smart_sap_core()->set('tax_artist_plural', strip_tags(stripslashes($sap['tax_artist_plural'])));
            smart_sap_core()->set('tax_artist_slug_single', strip_tags(stripslashes($sap['tax_artist_slug_single'])));
            smart_sap_core()->set('tax_artist_slug_archive', strip_tags(stripslashes($sap['tax_artist_slug_archive'])));

            smart_sap_core()->save();

            wp_redirect('edit.php?post_type=sapplaylist&page=smart-audio-playlists-advanced&settings-updated=true');
            exit;
        }        
    }

    public function save_playlist($post_ID, $post, $update) {
        if (isset($_POST['sap'])) {
            $playlist = $_POST['sap'];

            $reset_playlist = false;
            $reset_tracks = false;

            if (isset($playlist['reset'])) {
                $reset_playlist = isset($playlist['reset']['playlist_statistics']);
                $reset_tracks = isset($playlist['reset']['tracks_statistics']);
            }

            $processor = $this->process_tracks($playlist['tracks'], $reset_tracks);
            $playlist['meta']['tracks'] = $processor['tracks'];
            $playlist['meta']['total_time'] = $processor['time'];

            sap_playlist($post_ID)->update($playlist['meta']);

            if ($reset_playlist) {
                sap_playlist($post_ID)->reset();
            }
        }
    }

    public function process_tracks($raw, $_reset = false) {
        $tracks = array();
        $ids = array();
        $time = 0;

        $processor = smart_sap_core()->meta_processor(array(
            'int' => array('id', 'post', 'image'),
            'string' => array('length', 'type', 'url'),
            'html' => array(), 'bool' => array()
        ), 'track');

        foreach ($raw as $key => $obj) {
            foreach ($obj as $id => $value) {
                $id = intval($id);

                if ($id > 0) {
                    if (in_array($key, $processor['int'])) {
                        $value = intval($value);
                    } else if (in_array($key, $processor['string'])) {
                        $value = trim(strip_tags(stripslashes($value)));
                    } else if (in_array($key, $processor['html'])) {
                        $value = stripslashes($value);
                    }

                    $tracks[$id][$key] = $value;
                }
            }
        }

        $tracks = array_values($tracks);

        foreach ($tracks as $track) {
            if ($track['type'] == 'remote' && $track['url'] == '') {
                continue;
            }

            if ($track['id'] == 0 && $track['type'] == 'local') {
                $seek = get_post_meta($track['post'], '_sap_track_id', true);

                if ($seek != '' && is_numeric($seek)) {
                    $track['id'] = intval($seek);
                }
            }

            if ($track['id'] == 0) {
                $track['id'] = wp_insert_post(array(
                    'post_type' => 'saptrack',
                    'post_status' => 'publish',
                    'post_title' => $track['title']
                ));

                if ($track['type'] == 'local') {
                    update_post_meta($track['post'], '_sap_track_id', $track['id']);
                }
            }

            $id = $track['id'];
            unset($track['id']);

            if ($track['type'] == 'local') {
                $meta = wp_get_attachment_metadata($track['post']);
                $time+= intval($meta['length']) / 1000;
                $track['length'] = $meta['length_formatted'];
            } else if ($track['type'] == 'remote') {
                $parts = explode(':', $track['length']);

                if (count($parts) == 2) {
                    if (is_numeric($parts[0]) && is_numeric($parts[1])) {
                        $track['length'] = $parts[0].':'.$parts[1];

                        $time+= $parts[0] * 60 + $parts[1];
                    } else {
                        $track['length'] = '00:00';
                    }
                } else {
                    $track['length'] = '00:00';
                }
            }

            $t = sap_track($id);
            foreach ($track as $key => $val) {
                $t->$key = $val;
            }

            $t->save();

            if ($_reset) {
                $t->reset();
            }

            $ids[] = $id;
        }

        return array('tracks' => $ids, 'time' => $time);
    }

    public function meta_playlist_heading() {
        global $post_ID;

        $heading = sap_playlist($post_ID)->heading;

        wp_editor($heading, 'sap-heading', array(
            'textarea_rows' => 6,
            'media_buttons' => false,
            'textarea_name' => 'sap[meta][heading]'
        ));
    }

    public function meta_playlist_about() {
        
    }

    public function meta_playlist_info() {
        include(SAP_PATH.'forms/meta/info.php');
    }

    public function meta_playlist_meta() {
        include(SAP_PATH.'forms/meta/meta.php');
    }

    public function meta_playlist_tracks() {
        include(SAP_PATH.'forms/meta/tracks.php');
    }

    public function meta_playlist_details() {
        include(SAP_PATH.'forms/meta/details.php');
    }

    public function meta_playlist_settings() {
        include(SAP_PATH.'forms/meta/settings.php');
    }

    public function meta_playlist_stats() {
        include(SAP_PATH.'forms/meta/stats.php');
    }

    public function menu_settings() {
        global $sap_core_loader;

        $settings = $sap_core_loader->settings;

        include(SAP_PATH.'forms/index.php');
    }

    public function menu_about() {
        global $sap_core_loader;

        $about = $sap_core_loader->settings;

        include(SAP_PATH.'forms/index.php');
    }
}

global $sap_core_admin;
$sap_core_admin = new sap_admin();

?>