<?php

if (!defined('ABSPATH')) exit;

class sap_tracks_log extends WP_List_Table {
    public $playlist = 0;

    function __construct($args = array()) {
        $args = wp_parse_args($args, array(
            'plural' => 'tracks',
            'singular' => 'track',
            'ajax' => false,
            'screen' => null,
        ));

        $args['plural'] = sanitize_key($args['plural']);
        $args['singular'] = sanitize_key($args['singular']);

        $this->_args = $args;
    }

    function get_table_classes() {
        return array('widefat', 'fixed', $this->_args['plural'], 'sap-tracks-log');
    }

    function get_columns() {
	return array(
            'id' => __("ID", "smart-audio-playlists"),
            'track' => __("Track", "smart-audio-playlists"),
            'playtime' => __("Playtime", "smart-audio-playlists"),
            'count' => __("Played", "smart-audio-playlists"),
            'time' => '<span title="Time" class="dashicons dashicons-clock"></span>',
            'played' => __("Date", "smart-audio-playlists")
	);
    }

    function single_row($item) {
        static $row_class = '';
        $row_class = $row_class == '' ? ' alternate' : '';

        echo '<tr class="'.$row_class.'">';
        $this->single_row_columns($item);
        echo '</tr>';
    }

    function column_count($item) {
        return $item->plays.' '._n("time", "times", $item->plays, "smart-audio-playlists");
    }

    function column_playtime($item) {
        $min = floor($item->timer / 60);
        $sec = $item->timer - $min * 60;

        $min = $min < 10 ? '0'.$min : $min;
        $sec = $sec < 10 ? '0'.$sec : $sec;

        return $min.':'.$sec;
    }

    function column_track($item) {
        return $item->post_title;
    }

    function column_time($item) {
        return sap_track($item->post_id)->length;
    }

    function column_default($item, $column_name) {
        return $item->$column_name;
    }

    function prepare_items() {
        global $wpdb;

        $this->_column_headers = array($this->get_columns(), array(), $this->get_sortable_columns());

        $table = $wpdb->prefix.'sap_play_log';

        $query_where = array("l.type = 'track'", "l.plays > 0", "l.parent_id = ".$this->playlist);
        $query_subscribers = "SELECT l.*, p.post_title FROM ".$table." l";
        $query_subscribers.= " INNER JOIN ".$wpdb->posts." p ON p.ID = l.post_id";

        if (!empty($query_where)) {
            $query_subscribers.= ' WHERE '.join(' AND ', $query_where);
        }

        $this->items = $wpdb->get_results($query_subscribers);
    }
}

?>