<?php

if (!defined('ABSPATH')) exit;

class sap_track_core {
    public $id;
    public $meta = array(
        'type' => 'local',
        'url' => '',
        'length' => '00:00',
        'post' => 0,
        'image' => 0
    );

    function __construct($id = 0) {
        $this->id = $id;

        $this->meta = array_merge($this->meta, smart_sap_core()->meta_fields('track'));

        $values = get_post_meta($this->id, '_sap_track_information', true);

        $this->meta = wp_parse_args($values, $this->meta);
    }

    public static function instance($id) {
        static $sap_tracks = array();

        if (!isset($sap_tracks[$id])) {
            $sap_tracks[$id] = new sap_track_core($id);
        }

        return $sap_tracks[$id];
    }

    public function __get($name) {
        if (isset($this->meta[$name])) {
            return $this->meta[$name];
        } else {
            return '';
        }
    }

    public function __set($name, $value) {
        $this->meta[$name] = $value;
    }

    public function save() {
        update_post_meta($this->id, '_sap_track_information', $this->meta);
    }

    public function stats() {
        $defaults = array(
            '_day' => -1,
            '_week' => -1,
            '_month' => -1,
            'played_total' => 0,
            'played_day' => 0,
            'played_week' => 0,
            'played_month' => 0,
            'last_played' => 0
        );

        $values = array();
        if ($this->id > 0) {
            $values = get_post_meta($this->id, '_sap_track_statistics', true);
        }

        $stats = wp_parse_args($values, $defaults);

        $day = date('j');
        $week = date('W');
        $month = date('m');

        if ($stats['_day'] != $day) {
            $stats['_day'] = $day;
            $stats['played_day'] = 0;
        }

        if ($stats['_week'] != $week) {
            $stats['_week'] = $week;
            $stats['played_week'] = 0;
        }

        if ($stats['_month'] != $month) {
            $stats['_month'] = $month;
            $stats['played_month'] = 0;
        }

        return $stats;
    }

    public function reset() {
        delete_post_meta($this->id, '_sap_track_statistics');
    }

    public function played() {
        $stats = $this->stats();

        $day = date('j');
        $week = date('W');
        $month = date('m');

        if ($stats['_day'] != $day) {
            $stats['_day'] = $day;
            $stats['played_day'] = 0;
        }

        if ($stats['_week'] != $week) {
            $stats['_week'] = $week;
            $stats['played_week'] = 0;
        }

        if ($stats['_month'] != $month) {
            $stats['_month'] = $month;
            $stats['played_month'] = 0;
        }

        $stats['played_day']++;
        $stats['played_week']++;
        $stats['played_month']++;
        $stats['played_total']++;
        $stats['last_played'] = time();

        do_action('sap_played_track', $this->id, $stats);

        update_post_meta($this->id, '_sap_track_statistics', $stats);
    }

    public function log_add($log_id) {
        global $wpdb;

        $find = "select id, plays from ".$wpdb->prefix."sap_play_log where parent_id = ".$log_id." and post_id = ".$this->id;
        $logged = $wpdb->get_row($find);

        if (is_null($logged)) {
            $entry = array(
                'type' => 'track',
                'source' => 'inherit',
                'post_id' => $this->id,
                'parent_id' => $log_id,
                'user_id' => get_current_user_id(),
                'played' => current_time('mysql'),
                'plays' => 1,
                'ip' => smart_sap_core()->ip()
            );

            $wpdb->insert($wpdb->prefix.'sap_play_log', $entry);
        } else {
            $entry = array('plays' => intval($logged->plays) + 1);
            $update = array('id' => $logged->id);

            $wpdb->update($wpdb->prefix.'sap_play_log', $entry, $update);
        }
    }

    public function thumb_id() {
        if ($this->image > 0) {
            return $this->image;
        } else {
            if ($this->type == 'remote') {
                return 0;
            } else {
                $id = get_post_thumbnail_id($this->post);

                if ($id == '') {
                    return 0;
                } else {
                    return intval($id);
                }
            }
        }
    }
}

function sap_track($id = 0) {
    return sap_track_core::instance($id);
}

?>