<?php

if (!defined('ABSPATH')) exit;

class sap_widget extends WP_Widget {
    public $_widget_name = '';
    public $_widget_description = '';
    public $_widget_domain = 'sap_widget';
    public $_widget_code = 'widget';
    public $_widget_base = 'sap_widget';

    public $defaults = array(
        'title' => '',
        '_display' => 'all',
        '_class' => ''
    );

    function __construct($id_base = false, $name, $widget_options = array(), $control_options = array()) {
        $widget_options = empty($widget_options) ? array('classname' => 'cls_'.$this->_widget_base, 'description' => $this->_widget_description) : $widget_options;
        $control_options = empty($control_options) ? array('width' => 400) : $control_options;

        parent::__construct($this->_widget_base, $this->_widget_name, $widget_options, $control_options);
    }

    public function widget_id($args) {
        $this->widget_id = str_replace(array('-', '_'), array('', ''), $args['widget_id']);
    }

    public function widget($args, $instance) {
        $args = apply_filters($this->filter('arguments'), $args, $this);

        extract($args, EXTR_SKIP);

        $this->widget_id($args);

        $instance = wp_parse_args((array)$instance, $this->defaults);

        if ($this->visible($instance)) {
            $results = $this->widget_results($instance, $args);

            echo $before_widget;

            if (isset($instance['title']) && $instance['title'] != '') {
                echo $before_title;
                echo $this->widget_title($instance, $args);
                echo $after_title;
            }

            $this->widget_render($results, $instance, $args);

            echo $after_widget;
        }
    }

    function is_current_user_roles($roles = array()) {
        global $current_user;
        $roles = (array)$roles;

        if (is_array($current_user->roles) && !empty($roles)) {
            $match = array_intersect($roles, $current_user->roles);
            return !empty($match);
        } else {
            return false;
        }
    }

    public function visible($instance) {
        $visible = $this->is_visible($instance);

        if ($visible && isset($instance['_display'])) {
            $logged = is_user_logged_in();

            $role = substr($instance['_display'], 0, 5) == 'role:' ? substr($instance['_display'], 5) : false;
            $cap = substr($instance['_display'], 0, 4) == 'cap:' ? substr($instance['_display'], 4) : false;

            if ($role === false && $cap === false) {
                if ($instance['_display'] == 'all' || ($instance['_display'] == 'user' && $logged) || ($instance['_display'] == 'visitor' && !$logged)) {
                    $visible = true;
                } else {
                    $visible = false;
                }
            } else if ($role === false) {
                $visible = current_user_can($cap);
            } else {
                $visible = $this->is_current_user_roles($role);
            }
        }

        $visible = apply_filters($this->_widget_domain.'_visibility', $visible, $this, $instance);
        $visible = apply_filters($this->filter('visibility'), $visible, $this, $instance);

        return $visible;
    }

    public function filter($name = 'results') {
        return $this->_widget_base.'_'.$name.'_'.str_replace('-', '', $this->_widget_name);
    }

    public function is_visible($instance) {
        return true;
    }

    public function display_select_options_grouped($values, $selected = '', $name = '', $id = '', $class = '', $style = '', $multi = false, $echo = true) {
        $render = sap_render_select_grouped($values, array('selected' => $selected, 
            'name' => $name, 'id' => $id, 'class' => $class, 'style' => $style, 
            'multi' => $multi, 'echo' => false));

        if ($echo) {
            echo $render;
        } else {
            return $render;
        }
    }

    public function display_select_options($values, $selected = '', $name = '', $id = '', $class = '', $style = '', $multi = false, $echo = true) {
        $render = sap_render_select($values, array('selected' => $selected, 
            'name' => $name, 'id' => $id, 'class' => $class, 'style' => $style, 
            'multi' => $multi, 'echo' => false));

        if ($echo) {
            echo $render;
        } else {
            return $render;
        }
    }

    public function widget_title($instance, $args) {
        return $instance['title'];
    }

    public function widget_results($instance, $args) {
        return null;
    }
}

?>