(function($) {
    $.extend(mejs.MepDefaults, {
        nextText: "Next Track",
        prevText: "Previous Track"
    });

    $.extend(MediaElementPlayer.prototype, {
        buildprevtrack: function(player, controls, layers, media) {
            var t = this;

            var prevTrack = $('<div class="mejs-button mejs-prevtrack-button mejs-prevtrack">' +
                '<button type="button" aria-controls="' + player.id + '" title="' + player.options.prevText + '"></button>' +
                '</div>').appendTo(controls);

            t.prevTrack = t.controls.find(".mejs-prevtrack-button");
        },
        buildnexttrack: function(player, controls, layers, media) {
            var t = this;

            var nextTrack = $('<div class="mejs-button mejs-nexttrack-button mejs-nexttrack">' +
                '<button type="button" aria-controls="' + player.id + '" title="' + player.options.nextText + '"></button>' +
                '</div>').appendTo(controls);

            t.nextTrack = t.controls.find(".mejs-nexttrack-button");
        }
    });
})(mejs.$);

/*globals window, document, jQuery, _, Backbone, _wpmejsSettings */
(function($, _, Backbone) {
    "use strict";

    var sapPlaylistView = Backbone.View.extend({
        initialize : function(options) {
            this.timer = false;
            this.totalPlayed = 0;
            this.trackActive = 0;
            this.trackPlayed = 0;
            this.index = 0;
            this.autoloaded = true;
            this.data = options.metadata || $.parseJSON(this.$("script").html());

            this.startAjax();

            this.settings = {
                audioWidth: "100%",
                startVolume: this.data.volume,
                iPadUseNativeControls: this.data.native_ipad,
                iPhoneUseNativeControls: this.data.native_iphone,
                AndroidUseNativeControls: this.data.native_android,
                features: this.data.features
            };

            this.playerNode = $("<audio>");

            if (this.data.skin !== "") {
                this.playerNode.attr("class", this.data.skin);
            }

            this.tracks = new Backbone.Collection(this.data.tracks);
            this.current = this.tracks.first();

            this.trackActive = this.current.get("track");

            this.currentTemplate = wp.template("sap-playlist-current-item");
            this.currentNode = this.$(".sap-playlist-current-item");

            this.renderCurrent();

            this.$(".sap-playlist-header").append(this.playerNode);

            if (this.data.tracklist) {
                this.itemTemplate = wp.template("sap-playlist-item");
                this.playingClass = "sap-playlist-playing";
                this.renderTracks();
            }

            this.playerNode.attr("src", this.current.get("src"));

            _.bindAll(this, "endAjax", "bindPlayer", "bindResetPlayer", "prev", "next", "setPlayer", "pause", "count", "play", "ended", "clickTrack", "tracklistOpen", "tracklistClose");

            if (!_.isUndefined(window._wpmejsSettings)) {
                this.settings.pluginPath = _wpmejsSettings.pluginPath;
            }

            this.settings.success = this.bindPlayer;
            this.setPlayer();

            if (this.isMobile()) {
                $(".sap-playlist-item-played, .sap-playlist-item-download").show();
            }

            $(window).bind("beforeunload", this.endAjax);
        },

        isMobile: function() {
            var devices = "3ds|android|bada|bb10|hpwos|iemobile|kindle fire|opera mini|opera mobi|opera tablet|rim|silk|wiiu|ipad|ipod|iphone";
            var devicesRegEx = new RegExp(devices, "gi");

            return devicesRegEx.test(navigator.userAgent);
        },

        bindPlayer : function(mejs) {
            this.player = mejs;

            this.player.addEventListener("ended", this.ended);
            this.player.addEventListener("play", this.play);
        },

        bindResetPlayer : function(mejs) {
            this.bindPlayer(mejs);
            this.playCurrentSrc();
        },

        setPlayer: function() {
            if (this._player) {
                this._player.pause();
                this._player.remove();
                this.playerNode = $("<audio>");
                this.playerNode.attr("src", this.current.get("src"));
                this.settings.success = this.bindResetPlayer;
            }

            this._player = new MediaElementPlayer(this.playerNode.get(0), this.settings);

            if (sap_playlist_data.wpv === "38") {
                setTimeout(function(){
                    $(".sap-playlist-player .mejs-controls div.mejs-horizontal-volume-slider").css("width", "58px");
                }, 1000);
            }

            if (this.data.autoplay) {
                this.playCurrentSrc();
            }
        },

        startAjax : function() {
            var $setlog = this;

            $.ajax({
                type: "POST", data: {
                    playlist: this.data.playlist,
                    source: this.data.source
                }, dataType: "json",
                success: function(json) {
                    if (json.id > 0) {
                        $setlog.data.log = json.id;
                    }
                },
                url: sap_playlist_data.ajax + "?action=sap_log_playlist"
            });
        },

        endAjax : function() {
            if (this.totalPlayed > 0) {
                this.player.pause();

                $.ajax({
                    type: "POST", data: {
                        log: this.data.log, 
                        listTime: this.totalPlayed,
                        prevId: this.trackActive,
                        prevTime: this.trackPlayed
                    }, async: false,
                    url: sap_playlist_data.ajax + "?action=sap_ended_play"
                });
            }
        },

        playAjax : function() {
            this.autoloaded = false;

            $.ajax({
                type: "POST", data: {
                    track: this.current.get("track"), 
                    log: this.data.log, 
                    played: this.current.get("plays"), 
                    listTime: this.totalPlayed,
                    prevId: this.trackActive,
                    prevTime: this.trackPlayed
                },
                url: sap_playlist_data.ajax + "?action=sap_track_played"
            });

            this.current.set("plays", 1);
            this.tracks.at(this.index).set("plays", 1);
        },

        playCurrentSrc : function() {
            this.renderCurrent();

            this.playAjax();

            this.trackActive = this.current.get("track");
            this.trackPlayed = 0;

            this.player.setSrc(this.playerNode.attr("src"));
            this.player.load();
            this.player.play();
        },

        renderCurrent : function() {
            if (this.data.metadata) {
                if (!this.data.images) {
                    this.current.set("image", false);
                }

                this.currentNode.html(this.currentTemplate(this.current.toJSON()));
            }
        },

        renderTracks : function() {
            var self = this, 
                i = 1, 
                tracklist = this.$(".sap-playlist-tracks");

            this.tracks.each(function(model) {
                if (!self.data.images) {
                    model.set("image", false);
                }

                model.set("artists", self.data.artists);
                model.set("index", self.data.tracknumbers ? i : false);
                tracklist.append(self.itemTemplate(model.toJSON()));

                i+= 1;
            });

            this.$(".sap-playlist-item").eq(0).addClass(this.playingClass);
        },

        events : {
            "click .sap-playlist-item a.sap-playlist-item-play" : "clickTrack",
            "click .sap-playlist-item .sap-playlist-item-played a" : "clickStats",
            "click .sap-playlist-next" : "next",
            "click .sap-playlist-prev" : "prev",
            "click .sap-tracklist-open" : "tracklistOpen",
            "click .sap-tracklist-close" : "tracklistClose",
            "click .mejs-nexttrack-button" : "next",
            "click .mejs-prevtrack-button" : "prev"
        },

        tracklistOpen : function(e) {
            e.preventDefault();

            this.$(".sap-tracklist-open").parent().hide();
            this.$(".sap-playlist-tracks").slideDown();
            this.$(".sap-tracklist-close").parent().delay(300).show(0);
        },

        tracklistClose : function(e) {
            e.preventDefault();

            this.$(".sap-tracklist-close").parent().hide();
            this.$(".sap-playlist-tracks").slideUp();
            this.$(".sap-tracklist-open").parent().delay(300).show(0);
        },

        clickStats : function(e) {
            e.preventDefault();

            $(e.currentTarget).parent().next().slideToggle();
        },

        clickTrack : function(e) {
            e.preventDefault();

            this.autoloaded = false;

            this.index = this.$(".sap-playlist-item").index($(e.currentTarget).parent());
            this.setCurrent();
        },

        count : function() {
            this.totalPlayed++;
            this.trackPlayed++;
        },

        play : function() {
            if (this.autoloaded) {
                this.playAjax();
            }

            this.timer = setInterval(this.count, 1000);
        },

        pause : function() {
            clearInterval(this.timer);
        },

        ended : function() {
            this.pause();

            if (this.index + 1 < this.tracks.length) {
                this.next();
            } else {
                this.index = 0;
                this.current = this.tracks.at(this.index);
                this.loadCurrent();
            }
        },

        next : function() {
            this.index = this.index + 1 >= this.tracks.length ? 0 : this.index + 1;
            this.setCurrent();
        },

        prev : function() {
            this.index = this.index - 1 < 0 ? this.tracks.length - 1 : this.index - 1;
            this.setCurrent();
        },

        loadCurrent : function() {
            var last = this.playerNode.attr("src"),
                current = this.current.get("src");

            if (last !== current) {
                this.playerNode.attr("src", current);
                this.playCurrentSrc();
                this.player.play();
            } else {
                if (this.player.paused) {
                    this.player.play();
                } else {
                    this.player.pause();
                }
            }
        },

        setCurrent : function() {
            this.pause();

            this.current = this.tracks.at(this.index);

            if (this.data.tracklist) {
                this.$(".sap-playlist-item")
                    .removeClass(this.playingClass)
                    .eq(this.index)
                    .addClass(this.playingClass);
            }

            this.loadCurrent();
        }
    });

    $(document).ready(function() {
        $(".sap-playlist-player").each(function() {
            return new sapPlaylistView({el: this});
        });
    });

    window.sapPlaylistView = sapPlaylistView;
}(jQuery, _, Backbone));
