<?php

/*
Plugin Name: Smart Audio Playlists (shared on wplocker.com)
Plugin URI: http://www.smartplugins.info/plugin/wordpress/smart-audio-playlists/
Description: Easy to use audio playlists manager with HTML5 based audio player and support for old browsers fallback.
Version: 2.0
Author: Milan Petrovic
Author URI: http://www.dev4press.com/

== Copyright ==
Copyright 2008 - 2014 Milan Petrovic (email: milan@gdragon.info)
*/

class sap_loader {
    public $enqueued_playlist = false;
    public $enqueued_skins = array();
    public $id_pool = 0;

    public $objects;

    public $types = array();
    public $editions = array();
    public $styles = array();
    public $skins = array();

    public $fields = array(
        'track' => array(),
        'playlist' => array()
    );
    
    public $settings = array();

    function __construct() {
        global $wp_version;

        define('SAP_WP_VERSION', intval(substr(str_replace('.', '', $wp_version), 0, 2)));
        define('SAP_WP_VERSION_MAJOR', substr($wp_version, 0, 3));

        if (SAP_WP_VERSION < 38) exit;

        $_dirname = trailingslashit(dirname(__FILE__));
        $_urlname = plugin_dir_url(__FILE__);

        define('SAP_PATH', $_dirname);
        define('SAP_URL', $_urlname);

        if (!defined('SAP_EOL')) {
            define('SAP_EOL', "\r\n");
        }

        require_once(SAP_PATH.'core/noerror.php');
        require_once(SAP_PATH.'core/defaults.php');
        require_once(SAP_PATH.'core/objects.php');
        require_once(SAP_PATH.'core/track.php');
        require_once(SAP_PATH.'core/playlist.php');

        if (is_admin()) {
            require_once(SAP_PATH.'core/functions.php');
            require_once(SAP_PATH.'core/admin.php');
        } else {
            require_once(SAP_PATH.'core/templates.php');
            require_once(SAP_PATH.'core/player.php');
        }

        if (defined('DOING_AJAX') && DOING_AJAX) {
            require_once(SAP_PATH.'core/ajax.php');
        }

        register_activation_hook(__FILE__, array(&$this, 'install_database'));

        add_action('init', array(&$this, 'init_plugin_objects'), 2);

        add_action('widgets_init', array(&$this, 'widgets_init'));
        add_action('plugins_loaded', array(&$this, 'plugins_loaded'));
        add_action('after_setup_theme', array(&$this, 'theme_loaded'));
        add_action('wp_enqueue_scripts', array(&$this, 'wp_enqueue_scripts_global'));

        add_action('generate_rewrite_rules', array(&$this, 'generate_rewrite_rules'), 8);
        add_filter('query_vars', array(&$this, 'query_vars'));
        add_action('wp', array(&$this, 'expand_wp_query'), 1000);

        add_filter('template_include', array(&$this, 'intercept_template_include'), 1000);
        add_filter('template_include', array(&$this, 'intercept_taxonomy_index'));

        if (!is_admin()) {
            add_action('pre_get_posts', array(&$this, 'playlists_per_page'));
        }

        add_action('after_setup_theme', array(&$this, 'theme_post_thumbnails'), 1000000);

        if (WP_DEBUG) {
            add_action('wp_head', array(&$this, 'header_debug'), 0);
        }

        add_filter('soo_scanner_exclude_like_postmeta', array(&$this, 'soo_scanner_like'));
    }

    public function soo_scanner_like($list) {
        $list[] = '_sap_%';

        return $list;
    }

    public function install_database() {
        require_once(SAP_PATH.'core/install.php');

        sap_install_database();
    }

    public function header_debug() {
        global $template;

        echo '<!-- Loaded Template: '.$template.' -->'.SAP_EOL;
    }

    public function theme_post_thumbnails() {
        if (!current_theme_supports('post-thumbnails', 'sapplaylist')) {
            if ($this->get('force_post_thumbnails')) {
                add_theme_support('post-thumbnails');
            }
        }
    }

    public function playlists_per_page(&$query) {
        if (is_post_type_archive('sapplaylist')) {
            if (!isset($query->query_vars['suppress_filters']) || $query->query_vars['suppress_filters'] === false) {
                $query->query_vars['posts_per_page'] = smart_sap_core()->get('archive_playlists_count');
            }
        }
    }

    public function intercept_taxonomy_index($template) {
        global $wp_query;

        $new_template = '';

        if ($wp_query->is_sap_taxonomy_index) {
            $templates = array();

            $templates[] = 'taxindex-sapplaylist-'.$wp_query->sap_taxonomy_index.'.php';
            $templates[] = 'taxindex-sapplaylist.php';

            $new_template = sap_locate_theme_template($templates);
        }

        if (!empty($new_template)) {
            $template = $new_template;
        }

        return apply_filters('sap_taxonomy_index_template', $template);
    }

    public function expand_wp_query() {
        global $wp_query;

        $wp_query->is_sap_taxonomy_index = false;
        $wp_query->is_sap_taxonomy_archive = false;
        $wp_query->sap_taxonomy_index = '';
        $wp_query->sap_taxonomies = array();

        if (get_query_var('sapindex') != '') {
            $wp_query->sap_taxonomy_index = get_query_var('sapindex');
            $wp_query->is_sap_taxonomy_index = true;
            $wp_query->is_post_type_archive = false;
            $wp_query->is_home = false;
        }

        if (is_tax() || is_tag()) {
            $cpt = get_query_var('post_type');

            if ($cpt == 'sapplaylist') {
                $wp_query->is_sap_taxonomy_archive = true;
                $wp_query->is_tag = false;
                $wp_query->is_tax = false;

                foreach ($wp_query->tax_query->queries as $tax) {
                    $wp_query->sap_taxonomies[] = $tax['taxonomy'];
                }
            }
        }
    }

    public function generate_rewrite_rules($wp_rewrite) {
        $rules = sap_objects::rewrite_posttype_date_archives('sapplaylist', $wp_rewrite);

        $taxonomies = array();
        if ($this->settings['cpt_playlist_tax_tag']) {
            $taxonomies[] = 'post_tag';
        }
        if ($this->settings['cpt_playlist_tax_genre']) {
            $taxonomies[] = 'sapgenre';
        }
        if ($this->settings['cpt_playlist_tax_artist']) {
            $taxonomies[] = 'sapartist';
        }

        if (!empty($taxonomies)) {
            $rules = array_merge(sap_objects::rewrite_posttype_taxonomies('sapplaylist', $taxonomies, $wp_rewrite), $rules);
        }

        if (!empty($rules)) {
            $wp_rewrite->rules = $rules + $wp_rewrite->rules;
        }
    }

    public function query_vars($qv) {
        $qv[] = 'sapindex';

        return $qv;
    }

    public function widgets_init() {
        if (apply_filters('sap_load_widgets', true)) {
            require_once(SAP_PATH.'core/widget.php');
            require_once(SAP_PATH.'widgets/playlist.php');
            require_once(SAP_PATH.'widgets/playlists.php');

            register_widget('sap_widget_playlist');
            register_widget('sap_widget_playlists');
        }
    }

    public function intercept_template_include($template) {
        if ($this->get('templates_override')) {
            $tpl = basename($template);

            if (is_singular('sapplaylist') && $tpl != 'single-sapplaylist.php') {
                $template = $this->locate_template('single-sapplaylist.php', $template);
            } else if (is_post_type_archive('sapplaylist')) {
                global $wp_query;

                if (!$wp_query->is_sap_taxonomy_index && $tpl != 'archive-sapplaylist.php') {
                    $template = $this->locate_template('archive-sapplaylist.php', $template);
                }
            }
        }

        return $template;
    }

    public function locate_template($tpl, $template) {
        $theme = get_template();

        if (file_exists(SAP_PATH.'themes/'.$theme.'/')) {
            $override = SAP_PATH.'themes/'.$theme.'/'.$tpl;
        } else {
            $override = SAP_PATH.'themes/'.$tpl;
        }

        if (!file_exists($override)) {
            $override = $template;
        }

        return $override;
    }

    public function plugins_loaded() {
        $this->init_plugin_settings();
        $this->init_plugin_translation();
    }

    public function theme_loaded() {
        $this->init_styles();
        $this->init_skins();
        $this->init_lists();
        $this->init_fields();
        $this->init_theme();

        do_action('sap_init_plugin_visuals');

        require_once(SAP_PATH.'core/override.php');
    }

    public function init_plugin_translation() {
        $this->l = get_locale();

        if(!empty($this->l)) {
            load_plugin_textdomain('smart-audio-playlists', false, 'smart-audio-playlists/languages');
        }
    }

    public function init_plugin_settings() {
        $_d = new sap_defaults();

        $this->settings = get_option('smart-audio-playlists');

        if (!is_array($this->settings)) {
            $this->settings = $_d->settings;
            $this->settings['show_install_screen'] = true;

            update_option('smart-audio-playlists', $this->settings);
        } else if ($this->settings['__build__'] != $_d->settings['__build__']) {
            $this->settings = $_d->upgrade($this->settings);
            $this->settings['show_update_screen'] = true;

            update_option('smart-audio-playlists', $this->settings);
        }

        define('SMART_AUDIO_PLAYER', $this->settings['__version__']);

        do_action('sap_init_plugin_settings');
    }

    public function init_plugin_objects() {
        add_shortcode('saplist', 'sap_playlist_shortcode');

        add_image_size('sap-playlist-image', 450, 450);
        add_image_size('sap-archive-image', 300, 300, true);
        add_image_size('sap-track-image', 100, 100, true);
        add_image_size('sap-widget-image', 450, 120, true);

        sap_objects::register_track();
        sap_objects::register_playlist();

        if (smart_sap_core()->get('cpt_playlist_tax_genre')) {
            sap_objects::register_genre();
        }

        if (smart_sap_core()->get('cpt_playlist_tax_artist')) {
            sap_objects::register_artist();
        }

        do_action('sap_init_plugin_objects');

        add_action('sap_playlist_scripts', array(&$this, 'wp_enqueue_scripts'));
    }

    public function init_styles() {
        $this->styles = apply_filters('sap_playlist_styles', array(
            'sap-player-light' => __("Standard Light", "smart-audio-playlists"),
            'sap-player-dark' => __("Standard Dark", "smart-audio-playlists"),
            'sap-player-light-red' => __("Standard Light Red", "smart-audio-playlists"),
            'sap-player-flat-light' => __("Flat Light", "smart-audio-playlists"),
            'sap-player-flat-dark' => __("Flat Dark", "smart-audio-playlists"),
            'sap-player-flat-blue' => __("Flat Blue", "smart-audio-playlists"),
            'sap-player-flat-red' => __("Flat Red", "smart-audio-playlists"),
            'sap-player-flat-yellow' => __("Flat Yellow", "smart-audio-playlists")
        ));
    }

    public function init_skins() {
        $this->skins = apply_filters('sap_playlist_skins', array(
            'default' => __("Default", "smart-audio-playlists"),
            'mejskin' => __("Daniel Tara MEJSkin", "smart-audio-playlists")
        ));
    }

    public function init_lists() {
        $this->types = apply_filters('sap_playlist_type', array(
            'n-a' => __("Not Applicable", "smart-audio-playlists"),
            'album' => __("Album", "smart-audio-playlists"),
            'single' => __("Single", "smart-audio-playlists"),
            'ep' => __("EP", "smart-audio-playlists")
        ));

        $this->editions = apply_filters('sap_playlist_editions', array(
            'n-a' => __("Not Applicable", "smart-audio-playlists"),
            'standard' => __("Standard", "smart-audio-playlists"),
            'special' => __("Special", "smart-audio-playlists"),
            'deluxe' => __("Deluxe", "smart-audio-playlists"),
            'limited' => __("Limited", "smart-audio-playlists")
        ));
    }

    public function init_fields() {
        $this->fields['track'] = apply_filters('sap_fields_track', array(
            'title' => array('name' => __("Title", "smart-audio-playlists"), 'type' => 'string'),
            'artist' => array('name' => __("Artist", "smart-audio-playlists"), 'type' => 'string'),
            'album' => array('name' => __("Album", "smart-audio-playlists"), 'type' => 'string'),
            'genre' => array('name' => __("Genre", "smart-audio-playlists"), 'type' => 'string'),
            'year' => array('name' => __("Year", "smart-audio-playlists"), 'type' => 'int'),
            'publisher' => array('name' => __("Publisher", "smart-audio-playlists"), 'type' => 'string')
        ));

        $this->fields['playlist'] = apply_filters('sap_fields_playlist', array(
            'artist' => array('name' => __("Artist", "smart-audio-playlists"), 'type' => 'string'),
            'album' => array('name' => __("Album", "smart-audio-playlists"), 'type' => 'string'),
            'publisher' => array('name' => __("Publisher", "smart-audio-playlists"), 'type' => 'string')
        ));
    }

    public function init_theme() {
        if (defined('GENESIS_SETTINGS_FIELD')) {
            require_once(SAP_PATH.'themes/genesis/genesis-compat.php');
        }
    }

    public function meta_processor($processor, $type) {
        foreach ($this->fields[$type] as $key => $obj) {
            $processor[$obj['type']][] = $key;
        }

        return $processor;
    }

    public function meta_fields($type) {
        $meta = array();

        foreach ($this->fields[$type] as $key => $obj) {
            $value = '';

            switch ($obj['type']) {
                default:
                case 'string':
                    $value = isset($obj['default']) ? $obj['default'] : '';
                    break;
                case 'int':
                    $value = isset($obj['default']) ? $obj['default'] : 0;
                    break;
                case 'bool':
                    $value = isset($obj['default']) ? $obj['default'] : false;
                    break;
            }

            $meta[$key] = $value;
        }

        return $meta;
    }

    public function install_or_update() {
        return $this->settings['show_install_screen'] || $this->settings['show_update_screen'];
    }

    public function get($name) {
        return $this->settings[$name];
    }

    public function set($name, $value) {
        $this->settings[$name] = $value;
    }

    public function save() {
        update_option('smart-audio-playlists', $this->settings);
    }

    public function player_controls() {
        $features =  array();

        if (smart_sap_core()->get('player_prevtrack')) { $features[] = 'prevtrack'; }

        $features[] = 'playpause';

        if (smart_sap_core()->get('player_nexttrack')) { $features[] = 'nexttrack'; }
        if (smart_sap_core()->get('player_current')) { $features[] = 'current'; }
        if (smart_sap_core()->get('player_progress')) { $features[] = 'progress'; }
        if (smart_sap_core()->get('player_duration')) { $features[] = 'duration'; }
        if (smart_sap_core()->get('player_volume')) { $features[] = 'volume'; }

        return $features;
    }

    public function arrange_player_controls($features) {
        if (end($features) == 'progress') {
            $list = array('progress');

            $max = count($features) - 2;
            for ($i = 0; $i < count($features) - 1; $i++) {
                if ($i < $max) {
                    array_unshift($list, $features[$i]);
                } else {
                    array_push($list, $features[$i]);
                }
            }

            return $list;
        } else {
            return $features;
        }
    }

    public function tax_archive_slug($tax) {
        if ($tax == 'sapgenre') {
            $tax = 'genre';
        } else if ($tax == 'sapartist') {
            $tax = 'artist';
        } else if ($tax == 'post_tag') {
            $tax = 'tag';
        }

        if (isset($this->settings['tax_'.$tax.'_slug_archive'])) {
            return $this->settings['tax_'.$tax.'_slug_archive'];
        } else {
            return '';
        }
    }

    public function wp_enqueue_scripts_global() {
        $is_debug = defined('SCRIPT_DEBUG') && SCRIPT_DEBUG;

        if ($this->settings['css_load_render']) {
            wp_enqueue_style('sap-render', ($is_debug ? SAP_URL.'css/render.css' : SAP_URL.'css/render.min.css'), array('dashicons'), $this->settings['__version__']);
        }

        if ($this->settings['css_load_theme']) {
            if (file_exists(TEMPLATEPATH.'/playlists.css')) {
                wp_enqueue_style('sap-theme-template', get_template_directory_uri().'/playlists.css', array('dashicons', 'sap-render'));
            }

            if (file_exists(STYLESHEETPATH.'/playlists.css')) {
                wp_enqueue_style('sap-theme-stylesheet', get_stylesheet_directory_uri().'/playlists.css', array('dashicons', 'sap-render'));
            }
        }
    }

    function ip() {
        if (isset($_SERVER['HTTP_X_FORWARDED_FOR'])) {
            $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
        } else {
            $ip = $_SERVER['REMOTE_ADDR'];
        }

        $ip = preg_replace('/[^0-9a-fA-F:., ]/', '', $ip);

        if ($ip == '::1') {
            $ip = '127.0.0.1';
        }

        return trim($ip);
    }

    public function wp_enqueue_scripts($skin) {
        if (!$this->enqueued_playlist) {
            $is_debug = defined('SCRIPT_DEBUG') && SCRIPT_DEBUG;

            if ($this->settings['css_load_playlist']) {
                wp_enqueue_style('sap-playlist', ($is_debug ? SAP_URL.'css/playlist.css' : SAP_URL.'css/playlist.min.css'), array('dashicons', 'mediaelement'), $this->settings['__version__']);
            }

            wp_enqueue_script('sap-playlist', ($is_debug ? SAP_URL.'js/playlist.js' : SAP_URL.'js/playlist.min.js'), array('jquery', 'wp-util', 'backbone', 'mediaelement'), $this->settings['__version__'], true);

            wp_localize_script('sap-playlist', 'sap_playlist_data', array(
                'ajax' => admin_url('admin-ajax.php'),
                'wpv' => SAP_WP_VERSION
            ));

            add_action('wp_footer', 'sap_script_playlist_templates');

            $this->enqueued_playlist = true;

            do_action('sap_enqueue_scripts');
        }

        if ($skin != '' && $skin != 'default') {
            if (!in_array($skin, $this->enqueued_skins)) {
                wp_enqueue_style('sap-skin-'.$skin, SAP_URL.'skins/'.$skin.'/mediaelementplayer.css', array('mediaelement'), $this->settings['__version__']);
                $this->enqueued_skins[] = $skin;
            }
        }
    }
}

global $sap_core_loader;
$sap_core_loader = new sap_loader();

function smart_sap_core() {
    global $sap_core_loader;
    return $sap_core_loader;
}

function sap_playlist_shortcode($atts = array()) {
    $initial_volume = number_format(smart_sap_core()->get('player_initial_volume') / 100, 1);

    if (isset($atts['volume'])) {
        $atts['volume'] = intval($atts['volume']);

        if ($atts['volume'] > 100) {
            $atts['volume'] = 100;
        } else if ($atts['volume'] < 0) {
            $atts['volume'] = 0;
        }

        $atts['volume'] = number_format($atts['volume'] / 100, 1);
    }

    $features = array('playpause');
    $all_features = array('current', 'progress', 'duration', 'volume');

    foreach ($all_features as $f) {
        if (isset($atts['player_'.$f])) {
            if ($atts['player_'.$f] == 1) {
                $features[] = $f;
            }
        } else {
            if (smart_sap_core()->get('player_'.$f)) {
                $features[] = $f;
            }
        }
    }

    $defaults = array('id' => 0, 'link' => 1, 'width' => '', 
        'align' => 'none', 'autoplay' => 0, 'hidden' => 0,
        'volume' => $initial_volume, 
        'metadata' => smart_sap_core()->get('player_visible_metadata') ? 1 : 0, 
        'style' => smart_sap_core()->get('player_style'), 
        'skin' => smart_sap_core()->get('player_skin'));
    $atts = shortcode_atts($defaults, $atts);

    $args = $atts;
    $args['link_playlist'] = $args['link'];
    $args['max_width'] = $args['width'];
    $args['playlist_class'] = 'sap-shortcode';
    $args['source'] = 'shortcode';
    $args['autoplay'] = $args['autoplay'] == 1;
    $args['trackstats'] = false;
    $args['features'] = $features;

    if ($args['align'] != 'none') {
        $args['playlist_class'].= ' sap-align-'.$args['align'];
    }

    if ($args['hidden'] == 1) {
        $args['tracklist_toggle'] = true;
        $args['tracklist_toggle_default'] = 'hide';
    }

    unset($args['link']);
    unset($args['width']);
    unset($args['align']);
    unset($args['hidden']);

    $id = intval($atts['id']);

    if ($id > 0) {
        $post = get_post($id);

        if ($post && $post->post_type == 'sapplaylist') {
            $id = $post->ID;
        } else {
            $id = 0;
        }
    }

    if ($id == 0) {
        $posts = get_posts(array('post_type' => 'sapplaylist', 'numberposts' => 1, 'orderby' => 'rand'));
        $id = $posts[0]->ID;
    }

    return sap_render_player($id, $args, false);
}

?>