<?php

if (have_posts()) {
    while (have_posts()) {
        the_post();

        global $post;

        $class = 'smart-audio-playlist sap-playlist-single '.join(' ',get_post_class());

        $playlist = sap_playlist(get_the_ID());

        $statistics = $playlist->stats();

        $terms_genres = sap_terms_list(get_the_ID(), 'sapgenre');
        $terms_artists = sap_terms_list(get_the_ID(), 'sapartist');
        $terms_tags = sap_terms_list(get_the_ID(), 'post_tag');

        $related = sap_get_related_playlists($playlist->id);

        $class = apply_filters('sap_template_single_classes', $class, 'sap-single-playlist-alt');

?>

<div class="<?php echo $class; ?>">
    <?php if (smart_sap_core()->get('templates_single_title')) { ?>
    <h1 class="sap-entry-title"><?php the_title(); ?></h1>
    <?php } ?>

    <div class="sap-entry-heading">
        <div class="sap-heading-media">
            <?php the_post_thumbnail('sap-playlist-image'); ?>

            <ul>
                <li>
                    <i class="dashicons <?php echo SAP_WP_VERSION == 38 ? 'dashicons-format-audio' : 'dashicons-playlist-audio'; ?>"></i>
                    <div>
                        <h5><?php _e("Length", "smart-audio-playlists"); ?></h5>
                        <?php echo $playlist->running_length(); ?>
                    </div>
                </li>
                <li>
                    <i class="dashicons dashicons-clock"></i>
                    <div>
                        <h5><?php _e("Time", "smart-audio-playlists"); ?></h5>
                        <?php echo $playlist->running_time(); ?>
                    </div>
                </li>
            </ul>
        </div>
        <div class="sap-heading-details">
            <ul>
                <?php if (!empty($terms_artists)) { ?>
                <li>
                    <i class="dashicons dashicons-groups"></i>
                    <div>
                        <h5><?php echo _n(smart_sap_core()->get('tax_artist_singular'), smart_sap_core()->get('tax_artist_plural'), count($terms_artists), "smart-audio-playlists"); ?></h5>
                        <?php echo join(', ', $terms_artists); ?>
                    </div>
                </li>
                <?php } ?>
                <li>
                    <i class="dashicons dashicons-calendar"></i>
                    <div>
                        <h5><?php _e("Released", "smart-audio-playlists"); ?></h5>
                        <?php echo $playlist->release_date(); ?>
                    </div>
                </li>
                <?php if (!empty($terms_genres)) { ?>
                <li>
                    <i class="dashicons dashicons-awards"></i>
                    <div>
                        <h5><?php echo _n(smart_sap_core()->get('tax_genre_singular'), smart_sap_core()->get('tax_genre_plural'), count($terms_genres), "smart-audio-playlists"); ?></h5>
                        <?php echo join(', ', $terms_genres); ?>
                    </div>
                </li>
                <?php } ?>
                <?php if (!empty($terms_tags)) { ?>
                <li>
                    <i class="dashicons dashicons-tag"></i>
                    <div>
                        <h5><?php _e("Tags", "smart-audio-playlists"); ?></h5>
                        <?php echo join(', ', $terms_tags); ?>
                    </div>
                </li>
                <?php } ?>
            </ul>

            <?php if ($playlist->heading != '') { ?>
                <div class="sap-heading-content">
                    <?php echo $playlist->heading_content(); ?>
                </div>
            <?php } ?>
        </div>
    </div>

    <div class="sap-entry-block sap-entry-player">
        <h2><?php _e("Player", "smart-audio-playlists"); ?></h2>
        <?php sap_render_player($playlist); ?>
    </div>

    <?php if ($post->post_content != '' && $playlist->show_about == 1) { ?>
        <div class="sap-entry-block sap-entry-about">
            <h2><?php _e("About", "smart-audio-playlists"); ?></h2>
            <div class="entry-content">
                <?php the_content(); ?>
            </div>
        </div>
    <?php } ?>

    <?php if ($playlist->show_statistics == 1) { ?>
    <div class="sap-entry-block sap-entry-stats">
        <h2><?php _e("Statistics", "smart-audio-playlists"); ?></h2>
        <ul>
            <li>
                <i class="dashicons dashicons-chart-bar"></i>
                <div>
                    <h5><?php _e("Total", "smart-audio-playlists"); ?></h5>
                    <?php

                        echo $statistics['played_total'];
                        echo ' '._n("time", "times", $statistics['played_total'], "smart-audio-playlists");

                    ?>
                </div>
            </li>
            <li>
                <i class="dashicons dashicons-chart-area"></i>
                <div>
                    <h5><?php _e("This Week", "smart-audio-playlists"); ?></h5>
                    <?php

                        echo $statistics['played_week'];
                        echo ' '._n("time", "times", $statistics['played_week'], "smart-audio-playlists");

                    ?>
                </div>
            </li>
            <li>
                <i class="dashicons dashicons-chart-area"></i>
                <div>
                    <h5><?php _e("This Month", "smart-audio-playlists"); ?></h5>
                    <?php

                        echo $statistics['played_month'];
                        echo ' '._n("time", "times", $statistics['played_month'], "smart-audio-playlists");

                    ?>
                </div>
            </li>
            <li>
                <i class="dashicons dashicons-clock"></i>
                <div>
                    <h5><?php _e("Last Played", "smart-audio-playlists"); ?></h5>
                    <?php

                        echo human_time_diff($statistics['last_played']);
                        echo ' '.__("ago", "smart-audio-playlists");

                    ?>
                </div>
            </li>
        </ul>
    </div>
    <?php } ?>

    <?php if (!empty($related)) { ?>
    <div class="sap-entry-block sap-entry-related">
        <h2><?php _e("Related", "smart-audio-playlists"); ?></h2>

        <ul>
            <?php foreach ($related as $rel) { ?>
            <li>
                <?php $plrel = sap_playlist($rel->ID); ?>
                <a href="<?php echo get_permalink($rel->ID); ?>"><?php echo get_the_post_thumbnail($rel->ID, 'thumbnail'); ?></a>
                <a href="<?php echo get_permalink($rel->ID); ?>"><strong><?php echo $plrel->artist; ?></strong> - <?php echo $plrel->album; ?></a>
            </li>
            <?php } ?>
        </ul>
    </div>
    <?php } ?>
</div>

<?php

        $playlist->played();

        comments_template( '', true );
    }
}

?>