<?php

if (!defined('ABSPATH')) exit;

class sap_widget_playlist extends sap_widget {
    public $_widget_code = 'playlist';
    public $_widget_base = 'sap_widget_playlist';

    public $defaults = array(
        'title' => '',
        '_display' => 'all',
        '_class' => '',
        'id' => 0,
        'hidden' => true,
        'link' => true,
        'autoplay' => false,
        'metadata' => true,
        'header' => 'default',
        'style' => 'sap-player-light',
        'skin' => 'default',
        'volume' => 100,
        'player_prevtrack' => true,
        'player_nexttrack' => true,
        'player_current' => false,
        'player_progress' => true,
        'player_duration' => false,
        'player_volume' => true
    );

    function __construct($id_base = false, $name = '', $widget_options = array(), $control_options = array()) {
        $this->_widget_description = __("Show player for selected playlist.", "smart-audio-playlists");
        $this->_widget_name = __("Smart", "smart-audio-playlists").': '.__("Playlist Player", "smart-audio-playlists");

        parent::__construct($this->_widget_base, $this->_widget_name, array(), array('width' => 250));
    }

    public function form($instance) {
        $instance = wp_parse_args((array)$instance, $this->defaults);

        include(SAP_PATH.'forms/widgets/title.php');
        include(SAP_PATH.'forms/widgets/playlist.php');
        include(SAP_PATH.'forms/widgets/display.php');
    }

    public function update($new_instance, $old_instance) {
        $instance = $old_instance;

        $instance['title'] = strip_tags(stripslashes($new_instance['title']));
        $instance['_display'] = strip_tags(stripslashes($new_instance['_display']));
        $instance['_class'] = strip_tags(stripslashes($new_instance['_class']));

        $instance['volume'] = intval($new_instance['volume']);

        if ($instance['volume'] > 100) {
            $instance['volume'] = 100;
        } else if ($instance['volume'] < 0) {
            $instance['volume'] = 0;
        }

        $instance['link'] = isset($new_instance['link']);
        $instance['hidden'] = isset($new_instance['hidden']);
        $instance['autoplay'] = isset($new_instance['autoplay']);
        $instance['metadata'] = isset($new_instance['metadata']);
        $instance['player_prevtrack'] = isset($new_instance['player_prevtrack']);
        $instance['player_nexttrack'] = isset($new_instance['player_nexttrack']);
        $instance['player_current'] = isset($new_instance['player_current']);
        $instance['player_progress'] = isset($new_instance['player_progress']);
        $instance['player_duration'] = isset($new_instance['player_duration']);
        $instance['player_volume'] = isset($new_instance['player_volume']);

        $instance['header'] = strip_tags(stripslashes($new_instance['header']));
        $instance['style'] = strip_tags(stripslashes($new_instance['style']));
        $instance['skin'] = strip_tags(stripslashes($new_instance['skin']));
        $instance['id'] = intval($new_instance['id']);

        return $instance;
    }

    public function header_list() {
        return array(
            'default' => __("Default", "smart-audio-playlists"),
            'banner' => __("Banner", "smart-audio-playlists"),
            'full' => __("Full Image", "smart-audio-playlists")
        );
    }
    
    public function widget_results($instance, $args) {
        if ($instance['id'] > 0) {
            $post = get_post($instance['id']);

            if ($post && $post->post_type == 'sapplaylist') {
                return $post;
            }
        }

        $posts = get_posts(array('post_type' => 'sapplaylist', 'numberposts' => 1, 'orderby' => 'rand'));
        return $posts[0];
    }

    public function widget_render($results, $instance, $widget_args) {
        $initial_volume = number_format($instance['volume'] / 100, 1);

        $args = array(
            'source' => 'widget',
            'autoplay' => $instance['autoplay'],
            'metadata' => $instance['metadata'],
            'link_playlist' => $instance['link'],
            'volume' => $initial_volume,
            'skin' => $instance['skin'],
            'style' => $instance['style'],
            'trackstats' => false,
            'download' => false,
            'playlist_class' => 'sap-player-widget',
            'features' => array()
        );

        if ($instance['player_prevtrack']) { $args['features'][] = 'prevtrack'; }

        $args['features'][] = 'playpause';

        if ($instance['player_nexttrack']) { $args['features'][] = 'nexttrack'; }
        if ($instance['player_current']) { $args['features'][] = 'current'; }
        if ($instance['player_progress']) { $args['features'][] = 'progress'; }
        if ($instance['player_duration']) { $args['features'][] = 'duration'; }
        if ($instance['player_volume']) { $args['features'][] = 'volume'; }

        if ($instance['header'] == 'banner') {
            $args['thumbnail'] = 'sap-widget-image';
            $args['playlist_class'].= ' sap-header-banner';
        } else if ($instance['header'] == 'full') {
            $args['thumbnail'] = 'sap-playlist-image';
            $args['playlist_class'].= ' sap-header-banner';
        }

        if ($instance['hidden']) {
            $args['tracklist_toggle'] = true;
            $args['tracklist_toggle_default'] = 'hide';
        } 

        $widget_id = $widget_args['widget_id'];

        $class = apply_filters('sap_template_widget_playlist_player_classes', 'sap-widget-playlist-player', $instance, $results, $widget_id);
        echo '<div class="'.$class.'">';
        do_action('sap_widget_playlist_player_before', $instance, $results, $widget_id);

        sap_render_player($results->ID, $args);

        do_action('sap_widget_playlist_player_after', $instance, $results, $widget_id);
        echo '</div>';
    }
}

?>