<?php

if (!defined('ABSPATH')) exit;

class sap_widget_playlists extends sap_widget {
    public $_widget_code = 'playlists';
    public $_widget_base = 'sap_widget_playlists';

    public $defaults = array(
        'title' => 'Playlists',
        '_display' => 'all',
        '_class' => '',
        'count' => 3,
        'template' => '',
        'sort_column' => 'date',
        'sort_order' => 'DESC'
    );

    function __construct($id_base = false, $name = '', $widget_options = array(), $control_options = array()) {
        $this->_widget_description = __("Show list with several playlists.", "smart-audio-playlists");
        $this->_widget_name = __("Smart", "smart-audio-playlists").': '.__("Playlists Lists", "smart-audio-playlists");

        parent::__construct($this->_widget_base, $this->_widget_name, array(), array('width' => 250));
    }

    public function form($instance) {
        $instance = wp_parse_args((array)$instance, $this->defaults);

        include(SAP_PATH.'forms/widgets/title.php');
        include(SAP_PATH.'forms/widgets/playlists.php');
        include(SAP_PATH.'forms/widgets/display.php');
    }

    public function update($new_instance, $old_instance) {
        $instance = $old_instance;

        $instance['title'] = strip_tags(stripslashes($new_instance['title']));
        $instance['_display'] = strip_tags(stripslashes($new_instance['_display']));
        $instance['_class'] = strip_tags(stripslashes($new_instance['_class']));

        $instance['count'] = intval($new_instance['count']);
        $instance['template'] = strip_tags(stripslashes($new_instance['template']));
        $instance['sort_order'] = strip_tags(stripslashes($new_instance['sort_order']));
        $instance['sort_column'] = strip_tags(stripslashes($new_instance['sort_column']));

        if ($instance['count'] == 0) {
            $instance['count'] = 3;
        }

        return $instance;
    }

    public function widget_results($instance, $args) {
        return get_posts(array('post_type' => 'sapplaylist', 'numberposts' => $instance['count'], 
            'order' => $instance['sort_order'], 'orderby' => $instance['sort_column']
        ));
    }

    public function widget_render($results, $instance, $widget_args) {
        $widget_id = $widget_args['widget_id'];

        $class = apply_filters('sap_template_widget_playlists_classes', 'sap-widget-playlists', $instance, $results, $widget_id);
        
        echo '<div class="'.$class.'">';

        foreach ($results as $post) {
            global $playlist;

            $playlist = $post;
            sap_template_part('widget', 'playlist', $instance['template']);
        }

        echo '</div>';
    }
}

?>